# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand



# Unit tests for countMatrix_2longDtf
test_that("countMatrix_2longDtf converts count matrix to long data frame", {
  # Sample count matrix
  list_var <- init_variable()
  mock_data <- mock_rnaseq(list_var, n_genes = 3, 2,2, 1)
  # Convert count matrix to long data frame
  dtf_countLong <- countMatrix_2longDtf(mock_data$counts)
  expect_true(is.character(dtf_countLong$sampleID))
  expect_true(is.character(dtf_countLong$geneID))
  expect_true(is.numeric(dtf_countLong$kij))
  expect_equal(unique(dtf_countLong$geneID), c("gene1", "gene2", "gene3"))
  expect_equal(unique(dtf_countLong$sampleID),c("myVariable1_1", "myVariable1_2", 
                                                "myVariable2_1", "myVariable2_2"))
})

# Unit tests for getColumnWithSampleID
test_that("getColumnWithSampleID returns column name with sampleID", {
  # dummy data
  list_var <- init_variable()
  mock_data <- mock_rnaseq(list_var, n_genes = 3, 2,2)
  dtf_countLong <- countMatrix_2longDtf(mock_data$counts)
  
  # Expected output
  expected_output <- "sampleID"
  
  # Get column name with sampleID
  column_name <- getColumnWithSampleID(dtf_countLong, mock_data$metadata)
  
  # Check if the output matches the expected output
  expect_identical(column_name, expected_output)
})

# Unit tests for prepareData2fit
test_that("prepareData2fit prepares data for fitting", {
  # dummy data
  list_var <- init_variable()
  mock_data <- mock_rnaseq(list_var, n_genes = 3, 2,2)
  
  # Prepare data for fitting
  data2fit <- prepareData2fit(mock_data$counts, mock_data$metadata, normalization = 'MRN')
  
  expect_true(is.character(data2fit$sampleID))
  expect_true(is.character(data2fit$geneID))
  expect_true(is.numeric(data2fit$kij))
  expect_equal(unique(data2fit$geneID), c("gene1", "gene2", "gene3"))
  expect_equal(unique(data2fit$sampleID),c("myVariable1_1", "myVariable1_2", 
                                                "myVariable2_1", "myVariable2_2"))
  
  # Generate mock data
  countMatrix <- suppressWarnings(matrix(c(1, 2, 3, 4), ncol = 10, nrow = 3))
  colnames(countMatrix) <- paste0("sample", 1:10)
  metadata <- data.frame(sampleID = colnames(countMatrix), condition = rep(c("A", "B"), each = 5))
  
  # Call the function with log transformation
  data2fit <- prepareData2fit(countMatrix, metadata, normalization = NULL,transform = "log(x)")
  
  # Test if log transformation has been applied correctly
  expect_equal(unique(data2fit$kij), log(c(1, 2, 3, 4)))
  
  # Call the function with sqrt transformation
  data2fit <- prepareData2fit(countMatrix, metadata, normalization = NULL , transform = "sqrt(x)")
  # Test if log transformation has been applied correctly
  expect_equal(unique(data2fit$kij), sqrt(c(1, 2, 3, 4)))
})


# Tests unitaires pour isValidExpression
test_that("isValidExpression correctly validates custom expressions", {
  # Test avec une expression valide
  expect_true(isValidExpression("x + 1"))
  
  # Test avec une expression invalide (non chaîne de caractères)
  expect_false(isValidExpression(123))
  
  # Test avec une expression invalide (liste de chaînes de caractères)
  expect_false(isValidExpression(c("x + 1", "y + 2")))
  
  # Test avec une expression invalide (syntaxe incorrecte)
  expect_false(isValidExpression("log(x +)"))
})

# Tests unitaires pour custom_matrix_transform
test_that("custom_matrix_transform correctly applies custom function to count matrix", {
  count_matrix <- matrix(1:9, nrow = 3)
  # Test avec une expression valide
  transformed_matrix <- custom_matrix_transform(count_matrix, "x + 1")
  expect_equal(transformed_matrix, count_matrix + 1)
  
  # Test avec une expression invalide
  expect_error(custom_matrix_transform(count_matrix, "log(x +)"))
})



# Test case 1: Normalization with positive counts
test_that("Median ratio normalization works for positive counts", {
  counts <- matrix(c(100, 200, 300, 1000, 1500, 2500), ncol = 2)
  normalized_counts <- medianRatioNormalization(counts)
  
  expected_normalized_counts <- matrix(c(288.6751 , 577.3503 , 866.0254 , 346.4102, 519.6152, 866.0254), ncol = 2)
  
  expect_equal(normalized_counts, expected_normalized_counts, tolerance = 1e-4)
})

# Test case 2: Normalization with zero counts
test_that("Median ratio normalization return error for zero counts", {
  counts <- matrix(c(0, 0, 0, 1000, 1500, 2500), ncol = 2)
  expect_error(medianRatioNormalization(counts))
  
})


# Test case 5: All-zero genes
test_that("Throws an error when all-zero genes are encountered", {
  counts <- matrix(c(0, 0, 0, 0, 0, 0), ncol = 2)
  expect_error(medianRatioNormalization(counts))
})



# Test case 1: Check if normalization is performed correctly
test_that("Normalized counts are calculated correctly", {
  # Create a mock count matrix
  counts_matrix <- matrix(c(10, 20, 30, 40, 50, 60), nrow = 3, byrow = TRUE)
  
  # Expected normalized counts
  expected_normalized <- matrix(c(11.54701, 34.64102, 57.73503, 17.32051, 34.64102, 51.96152), nrow = 3, byrow = F)
  
  # Perform normalization
  normalized_counts <- trimmedMeanMvaluesNormalization(counts_matrix)
  
  # Check if normalized counts match the expected values
  expect_equal(normalized_counts, expected_normalized, tolerance = 1e-2)
})

# Test case 2: Check if input matrix is unchanged when all counts are 0
test_that("Input matrix remains unchanged when all counts are 0", {
  # Create a mock count matrix with all counts as 0
  counts_matrix <- matrix(0, nrow = 3, ncol = 2)
  
  # Perform normalization
  normalized_counts <- trimmedMeanMvaluesNormalization(counts_matrix)
  
  # Check if input matrix and normalized matrix are identical
  expect_identical(normalized_counts, counts_matrix)
})


