# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand




# Test invalid data
test_that("is_truthLabels_valid returns TRUE for valid data with both labels present", {
  eval_data <- data.frame(description = c("A", "B", "C"), isDE = c(TRUE, FALSE, FALSE), effect = c("fixed", "fixed", "fixed"))
  result <- is_truthLabels_valid(eval_data)
  expect_equal(result, FALSE)
})

# Test valid data
test_that("is_truthLabels_valid returns FALSE for valid data with only one label present", {
eval_data <- data.frame(description = c("A", "A", "A"), isDE = c(TRUE, NA, FALSE), effect = c("fixed", "fixed", "fixed"))
  result <- is_truthLabels_valid(eval_data)
  expect_equal(result, TRUE)
})




# Tester isValidEvalInput avec des données valides
test_that("isValidEvalInput with valid input", {
  
    # Définir des données de test
    mock_obj <- list(settings = NULL, init = NULL, groundTruth = NULL, counts = NULL, metadata = NULL, scaling_factors = NULL)
    list_gene <- c("gene1", "gene2", "gene3")
    list_tmb <- list(glmmTMB1 = NULL, glmmTMB2 = NULL)
    dds <- NULL
    coeff_threshold <- 0.05
    alt_hypothesis <- "greaterAbs"
    alpha_risk <- 0.01
  
  expect_error(isValidEvalInput(mock_obj, list_gene, list_tmb, dds, coeff_threshold, alt_hypothesis, alpha_risk),
               "All elements in 'list_tmb' are NULL")

  # Tester isValidEvalInput avec list_tmb et dds à NULL
  expect_error(isValidEvalInput(mock_obj, list_gene, NULL, NULL, coeff_threshold, alt_hypothesis, alpha_risk),
               "Both 'list_tmb' and 'dds' are NULL. There is nothing to evaluate.")


  # Tester isValidEvalInput avec alt_hypothesis incorrect
  l_tmb <- list("model1" = glmmTMB::glmmTMB(mpg ~ hp + vs + am + (1|cyl), data = mtcars),
                 "model2" = glmmTMB::glmmTMB(mpg ~ hp + vs + am + (1|cyl), data = mtcars))
  expect_error(isValidEvalInput(mock_obj, list_gene, l_tmb, dds, coeff_threshold, "greaterThan", alpha_risk))
  
  # alpha risk incoret
  expect_error(isValidEvalInput(mock_obj, list_gene, l_tmb, dds, coeff_threshold, "greater", "incorrect_alpha_risk"))

  # coeff_threshold  incoret
  expect_error(isValidEvalInput(mock_obj, list_gene, l_tmb, dds, c(), "greater", alpha_risk))

   # mock_obj  incoret
  expect_error(isValidEvalInput(c(), list_gene, l_tmb, dds, coeff_threshold, "greater", alpha_risk))

  
  ## no error 
  expect_true(isValidEvalInput(mock_obj, list_gene, l_tmb, dds, coeff_threshold, "greater", alpha_risk))
})




test_that("evaluation_report returns correct output", {
  N_GENES <- 100
  MAX_REPLICATES <- 5
  MIN_REPLICATES <- 5
  input_var_list <- init_variable(name = "varA", mu = 10, sd = 0.1, level = 3)
  mock_data <- mock_rnaseq(input_var_list, N_GENES,
                           min_replicates = MIN_REPLICATES, max_replicates = MAX_REPLICATES)
  
  data2fit <- prepareData2fit(countMatrix = mock_data$counts, metadata = mock_data$metadata, normalization = 'MRN')
  l_res <- fitModelParallel(formula = kij ~ varA,
                            data = data2fit, group_by = "geneID",
                            family = glmmTMB::nbinom2(link = "log"), n.cores = 1)
  
  # Tests here
  report <- evaluation_report(list_tmb = l_res, 
                              dds = NULL, 
                              mock_obj = mock_data, 
                              coeff_threshold = 0.01, 
                              alt_hypothesis = "greater")
  
  expect_true(is.list(report))
  expect_true("data" %in% names(report))
  expect_true("identity" %in% names(report))
  expect_true("precision_recall" %in% names(report))
  expect_true("roc" %in% names(report))
  expect_true("counts" %in% names(report))
  expect_true("performances" %in% names(report))

  expect_error(evaluation_report(list_tmb = l_res, 
                    dds = NULL, 
                    mock_obj = NULL, # throw error 
                    coeff_threshold = 0.01, 
                    alt_hypothesis = "greater")  )
})




test_that("get_performances_metrics_obj returns correct output", {
  # Define the input data
  r2_params <- data.frame(from = c("Glm", "Hglm"),
                          description = c("query_1|Intercept", "query_1|Intercept", "query_1|varA",  "query_1|varA"),
                          R2 = c(0.9, 0.7))
  r2_dispersion <- data.frame(from = c("Glm", "Hglm"),
                               description = c("dispersion"),
                               R2 = c(0.8, 0.6))
  pr_obj <- list(byparams = list(pr_auc = data.frame(from = c("Glm", "Hglm"),
                                                      description = c("query_1|Intercept", "query_1|Intercept", "query_1|varA",  "query_1|varA"),
                                                      pr_auc = 0.9)),
                 aggregate = list(pr_auc = data.frame(from = c("Glm", "Hglm"),
                                                      pr_auc = 0.8)))
  roc_obj <- list(byparams = list(roc_auc = data.frame(from = c("Glm", "Hglm"),
                                                        description = c("query_1|Intercept", "query_1|Intercept", "query_1|varA",  "query_1|varA"),
                                                        roc_auc = 0.7)),
                  aggregate = list(roc_auc = data.frame(from = c("Glm", "Hglm"),
                                                        roc_auc = 0.6)))
  ml_metrics_obj <- list(byparams = data.frame(from = c("Glm", "Hglm"),
                                                description = c("query_1|Intercept", "query_1|Intercept", "query_1|varA", "query_1|varA"),
                                                accuracy = c(0.9, 0.8),
                                                recall = c(0.09, 0.88)),
                         aggregate = data.frame(from = c("Glm", "Hglm"),
                                                accuracy = c(0.7, 0.6), recall = c(0.1, 0.8)))
  r2_agg <- data.frame(from = c("Glm", "Hglm"),
                          RMSE = c(0.22, 0.55),
                          R2 = c(0.9, 0.7))
  
  # Call the function
  result <- get_performances_metrics_obj(r2_params, r2_agg, r2_dispersion,
                                          pr_obj, roc_obj, ml_metrics_obj)
  
  # Test the output
  expect_true(is.list(result))
  expect_true("byparams" %in% names(result))
  expect_true("aggregate" %in% names(result))
  expect_equal(nrow(result$byparams), 6)
  expect_equal(ncol(result$byparams), 7)
  expect_equal(nrow(result$aggregate), 2)
  expect_equal(ncol(result$aggregate), 7)
})





test_that("compute_metrics_summary returns correct output",{
  # Define input data
  dt <- data.frame(
    y_pred = c(TRUE, TRUE, FALSE, TRUE, FALSE, FALSE, TRUE, TRUE),
    isDE = c(TRUE, FALSE, TRUE, TRUE, FALSE, TRUE, FALSE, FALSE)
  )
  # Compute expected output
  acc_exp <- 0.375
  prec_exp <- 0.4
  rec_exp <- 0.5
  sens_exp <- 0.5
  spec_exp <- 0.25
  exp_df <- data.frame(
    accuracy = acc_exp,
    precision = prec_exp,
    recall = rec_exp,
    sensitivity = sens_exp,
    specificity = spec_exp
  )
  # Compute actual output with tested function
  act_df <- compute_metrics_summary(dt)
  # Test that compute_metrics_summary returns expected output
  expect_identical(act_df, exp_df)
})


test_that("get_ml_metrics_obj returns correct output", {
  # Define input data
  seed = 100
  evaldata_params <- data.frame(
    from = rep(c("Glm", "Hglm", "Hglm", "Glm"), 1000),
    description = rep(c("intercept", "varA", "intercept", "varA"), 1000),
    p.adj = sort(runif(1000), decreasing = T),
    isDE = sort(sample(c(TRUE, FALSE), 1000, replace = T)),
    effect = rep("fixed", 1000)
    )
 
  # Compute actual output with tested function
  act_output <- get_ml_metrics_obj(evaldata_params, alpha_risk = 0.05, col_param = "description")
  # Test actual output against expected output
  expect_identical(names(act_output$byparams), c( "from","description","accuracy","precision","recall","sensitivity","specificity"))
  expect_identical(names(act_output$aggregate), c( "from","accuracy","precision","recall","sensitivity","specificity"))

})







# Test get_eval_data_from_ltmb
test_that("get_eval_data_from_ltmb returns correct output", {
  
  input_var_list <- init_variable( name = "varA", mu = 3, sd = 2, level = 3) 
  
  ## -- Required parameters
  N_GENES = 3
  MIN_REPLICATES = 3
  MAX_REPLICATES = 3
  ########################
  
  ## -- simulate RNAseq data based on input_var_list, minimum input required
  ## -- number of replicate randomly defined between MIN_REP and MAX_REP
  mock_data <- mock_rnaseq(input_var_list, N_GENES,
                         min_replicates  = MIN_REPLICATES,
                         max_replicates = MAX_REPLICATES)
  ## -- data from simulation 
  count_matrix <- mock_data$counts
  metaData <- mock_data$metadata
  ##############################
  ## -- convert counts matrix and samples metadatas in a data frame for fitting
  data2fit = prepareData2fit(countMatrix = count_matrix,
                           metadata =  metaData,
                           normalization = NULL)
  l_tmb <- fitModelParallel(formula = kij ~ varA  ,
                          data = data2fit,
                          group_by = "geneID",
                          family = glmmTMB::nbinom2(link = "log"),
                          n.cores = 1)
 
  eval_data_ltmb <- get_eval_data_from_ltmb(l_tmb, mock_data, 0.27, 'greater')
  expect_is(eval_data_ltmb, "list")
  expect_named(eval_data_ltmb, c("modelparams", "modeldispersion"))
})

# Test get_eval_data_from_dds
test_that("get_eval_data_from_dds returns correct output", {
  
  input_var_list <- init_variable( name = "varA", mu = 3, sd = 2, level = 3) 
  
  ## -- Required parameters
  N_GENES = 100
  MIN_REPLICATES = 3
  MAX_REPLICATES = 3
  ########################
  
  ## -- simulate RNAseq data based on input_var_list, minimum input required
  ## -- number of replicate randomly defined between MIN_REP and MAX_REP
  mock_data <- mock_rnaseq(input_var_list, N_GENES,
                         min_replicates  = MIN_REPLICATES,
                         max_replicates = MAX_REPLICATES)
  ## -- data from simulation 
  count_matrix <- mock_data$counts
  metaData <- mock_data$metadata
  ##############################
  
  dds <- DESeq2::DESeqDataSetFromMatrix(count_matrix, colData = metaData, ~ varA )
  dds <- DESeq2::DESeq(dds)
  class(dds)
  eval_data_dds <- get_eval_data_from_dds(dds, mock_data, 0.27, "greater")
  
  expect_is(eval_data_dds, "list")
  expect_named(eval_data_dds, c("modelparams", "modeldispersion"))
})

# Test rbind_evaldata_tmb_dds
test_that("rbind_evaldata_tmb_dds returns correct output", {
  
  input_var_list <- init_variable( name = "varA", mu = 3, sd = 2, level = 3) 
  
  ## -- Required parameters
  N_GENES = 15
  MIN_REPLICATES = 3
  MAX_REPLICATES = 3
  BASAL_EXPR <- 3
  ########################
  
  ## -- simulate RNAseq data based on input_var_list, minimum input required
  ## -- number of replicate randomly defined between MIN_REP and MAX_REP
  mock_data <- mock_rnaseq(input_var_list, N_GENES,
                         min_replicates  = MIN_REPLICATES,
                         max_replicates = MAX_REPLICATES,
                         basal_expression = BASAL_EXPR)
  ## -- data from simulation 
  count_matrix <- mock_data$counts
  metaData <- mock_data$metadata
  ##############################
  ## -- convert counts matrix and samples metadatas in a data frame for fitting
  data2fit = prepareData2fit(countMatrix = count_matrix,
                           metadata =  metaData,
                           normalization = NULL)
  l_tmb <- fitModelParallel(formula = kij ~ varA  ,
                          data = data2fit,
                          group_by = "geneID",
                          family = glmmTMB::nbinom2(link = "log"),
                          n.cores = 1)
  dds <- DESeq2::DESeqDataSetFromMatrix(count_matrix, colData = metaData, ~ varA )
  dds <- DESeq2::DESeq(dds)
  
  eval_data_dds <- get_eval_data_from_dds(dds, mock_data, 0.27, "greater")
  eval_data_ltmb <- get_eval_data_from_ltmb(l_tmb, mock_data, 0.27, 'greater')
  
  combined_eval_data <- rbind_evaldata_tmb_dds(eval_data_ltmb, eval_data_dds)
  
  expect_is(combined_eval_data, "list")
  expect_named(combined_eval_data, c("modelparams", "modeldispersion"))
})

# Test rbind_model_params_and_dispersion
test_that("rbind_model_params_and_dispersion returns correct output", {
  
  input_var_list <- init_variable( name = "varA", mu = 3, sd = 2, level = 3) 
  
  ## -- Required parameters
  N_GENES = 100
  MIN_REPLICATES = 3
  MAX_REPLICATES = 3
  ########################
  
  ## -- simulate RNAseq data based on input_var_list, minimum input required
  ## -- number of replicate randomly defined between MIN_REP and MAX_REP
  mock_data <- mock_rnaseq(input_var_list, N_GENES,
                         min_replicates  = MIN_REPLICATES,
                         max_replicates = MAX_REPLICATES)
  ## -- data from simulation 
  count_matrix <- mock_data$counts
  metaData <- mock_data$metadata
  ##############################

  dds <- DESeq2::DESeqDataSetFromMatrix(count_matrix, colData = metaData, ~ varA )
  dds <- DESeq2::DESeq(dds)
  
  
  eval_data <- get_eval_data_from_dds(dds, mock_data, 0.27, "greater")
  
  combined_data <- rbind_model_params_and_dispersion(eval_data)
  
  expect_is(combined_data, "data.frame")
})

# Test get_eval_data
test_that("get_eval_data returns correct output", {
  
  input_var_list <- init_variable( name = "varA", mu = 0, sd = 1, level = 3) 
  
  ## -- Required parameters
  N_GENES = 50
  MIN_REPLICATES = 3
  MAX_REPLICATES = 3
  ########################
  
  ## -- simulate RNAseq data based on input_var_list, minimum input required
  ## -- number of replicate randomly defined between MIN_REP and MAX_REP
  mock_data <- mock_rnaseq(input_var_list, N_GENES,
                         min_replicates  = MIN_REPLICATES,
                         max_replicates = MAX_REPLICATES)
  ## -- data from simulation 
  count_matrix <- mock_data$counts
  metaData <- mock_data$metadata
  ##############################
  ## -- convert counts matrix and samples metadatas in a data frame for fitting
  data2fit = prepareData2fit(countMatrix = count_matrix,
                           metadata =  metaData,
                           normalization = NULL)
  l_tmb <- fitModelParallel(formula = kij ~ varA  ,
                          data = data2fit,
                          group_by = "geneID",
                          family = glmmTMB::nbinom2(link = "log"),
                          n.cores = 1)
  dds <- DESeq2::DESeqDataSetFromMatrix(count_matrix, colData = metaData, ~ varA )
  dds <- DESeq2::DESeq(dds)
  
  
  eval_data <- get_eval_data(l_tmb, dds, mock_data, 0.27, "greater")
  
  expect_is(eval_data, "list")
  expect_named(eval_data, c("modelparams", "modeldispersion"))
  expect_equal(unique(eval_data$modelparams$from), c("HTRfit", "DESeq2"))
  
  
  ## -- dds == NULL
  eval_data <- get_eval_data(l_tmb, NULL, mock_data, 0.27, "greater")
  expect_is(eval_data, "list")
  expect_named(eval_data, c("modelparams", "modeldispersion"))
  expect_equal(unique(eval_data$modelparams$from), c("HTRfit"))
  
  
    ## -- l_tmb == NULL
  eval_data <- get_eval_data(NULL, dds, mock_data, 0.27, "greater")
  expect_is(eval_data, "list")
  expect_named(eval_data, c("modelparams", "modeldispersion"))
  expect_equal(unique(eval_data$modelparams$from), c("DESeq2"))

})



